<?php

namespace Drupal\o365;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\Exception\ClientException;
use Microsoft\Graph\Graph;

/**
 * This is the GraphService.
 *
 * The main service that gets and sets data in Office 365.
 */
class GraphService {

  /**
   * Drupal\o365\AuthenticationServiceInterface definition.
   *
   * @var \Drupal\o365\AuthenticationServiceInterface
   */
  protected $authService;

  /**
   * The logger service.
   *
   * @var \Drupal\o365\O365LoggerServiceInterface
   */
  protected $messenger;

  /**
   * The cache baclend service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * The modules logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a new GraphService object.
   *
   * @param \Drupal\o365\AuthenticationServiceInterface $authenticationService
   *   The AuthenticationServiceInterface definition.
   * @param \Drupal\o365\O365LoggerServiceInterface $messenger
   *   The O365LoggerServiceInterface definition.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The CacheBackendInterface definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The logger channel factory. Used to log to the watchdog.
   */
  public function __construct(AuthenticationServiceInterface $authenticationService, O365LoggerServiceInterface $messenger, CacheBackendInterface $cacheBackend, LoggerChannelFactoryInterface $loggerChannelFactory) {
    $this->authService = $authenticationService;
    $this->messenger = $messenger;
    $this->cacheBackend = $cacheBackend;
    $this->logger = $loggerChannelFactory->get('o365');
  }

  /**
   * Get data from the MS GraphAPI.
   *
   * @param string $endpoint
   *   The graph endpoint we want data from.
   * @param string $type
   *   The type of request we want to do.
   * @param bool $raw
   *   This determines if we want a raw body or not.
   * @param string|bool $version
   *   The version of the graph api that is used.
   * @param string|bool $returnType
   *   The return type we want to get.
   * @param array $headers
   *   An array of headers to send.
   *
   * @return mixed
   *   The data retrieved from the Graph API.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException|\GuzzleHttp\Exception\GuzzleException
   */
  public function getGraphData($endpoint, $type = 'GET', $raw = FALSE, $version = FALSE, $returnType = FALSE, array $headers = []) {
    return $this->getData($endpoint, $type, $raw, $version, [], $returnType, $headers);
  }

  /**
   * Send data to the MS GraphAPI.
   *
   * @param string $endpoint
   *   The graph endpoint we want data from.
   * @param array $data
   *   Sent data.
   * @param string $type
   *   The type of request we want to do.
   * @param bool $raw
   *   This determines if we want a raw body or not.
   * @param string|bool $version
   *   The version of the graph api that is used.
   *
   * @return mixed
   *   The data retrieved from the Graph API.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException|\GuzzleHttp\Exception\GuzzleException
   */
  public function sendGraphData($endpoint, array $data = [], $type = 'POST', $raw = FALSE, $version = FALSE) {
    return $this->getData($endpoint, $type, $raw, $version, $data);
  }

  /**
   * Generic method to get or send data to the Graph endpoints.
   *
   * @param string $endpoint
   *   The graph endpoint we want data from.
   * @param string $type
   *   The type of request we want to do.
   * @param bool $raw
   *   This determines if we want a raw body or not.
   * @param string|bool $version
   *   The version of the graph api that is used.
   * @param array $data
   *   Sent data.
   * @param string|bool $returnType
   *   The return type we want to get.
   * @param array $headers
   *   An array of headers to send.
   *
   * @return array|false|string|null
   *   The retrieved data from the GRAPH API.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \GuzzleHttp\Exception\GuzzleException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   */
  private function getData($endpoint, $type = 'GET', $raw = FALSE, $version = FALSE, array $data = [], $returnType = FALSE, array $headers = []) {
    // Get the data from the Graph API.
    try {
      $graph = new Graph();

      if ($version) {
        $graph->setApiVersion($version);
      }

      $accessToken = $this->authService->getAccessToken();
      $graph->setAccessToken($accessToken);

      if ($type === 'POST') {
        /** @var \Microsoft\Graph\Http\GraphResponse $request */
        $request = $graph->createRequest($type, $endpoint)
          ->addHeaders($headers)
          ->attachBody($data)
          ->execute();
      }
      else {
        if ($returnType) {
          /** @var \Microsoft\Graph\Http\GraphRequest $request */
          $request = $graph->createRequest($type, $endpoint)
            ->addHeaders($headers)
            ->setReturnType($returnType);
          return $request->execute();
        }

        /** @var \Microsoft\Graph\Http\GraphRequest $request */
        $request = $graph->createRequest($type, $endpoint)
          ->addHeaders($headers)
          ->execute();
      }

      // Get the return values.
      $return = $request->getBody();
      if ($raw) {
        $return = $request->getRawBody();
      }

      return $return;
    }
    catch (ClientException $e) {
      // Log the error message.
      $this->logger->error($e->getMessage());
    }

    return FALSE;
  }

  /**
   * Get a collection of data.
   *
   * @param string $endpoint
   *   The graph endpoint we want data from.
   * @param false|string $returnType
   *   The return type we want to get.
   * @param false|string $version
   *   The version of the graph api that is used.
   *
   * @return false|\Microsoft\Graph\Http\GraphCollectionRequest
   *   The collection request.
   *
   * @throws \Drupal\Core\TempStore\TempStoreException
   * @throws \League\OAuth2\Client\Provider\Exception\IdentityProviderException
   * @throws \Microsoft\Graph\Exception\GraphException
   */
  public function getCollectionData($endpoint, $returnType = FALSE, $version = FALSE) {
    try {
      $graph = new Graph();

      if ($version) {
        $graph->setApiVersion($version);
      }

      $accessToken = $this->authService->getAccessToken();
      $graph->setAccessToken($accessToken);
      $request = $graph->createCollectionRequest('GET', $endpoint);

      if ($returnType) {
        $request->setReturnType($returnType);
      }

      return $request;
    }
    catch (ClientException $e) {
      // Do nothing.
    }

    return FALSE;
  }

  /**
   * Determine the office user ID.
   *
   * @return bool|string
   *   The ID or FALSE if not logged in via Office.
   */
  public function getCurrentUserId() {
    return $this->authService->checkForOfficeLogin();
  }

}
